"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const core_1 = require("@angular-devkit/core");
const schematics_utilities_1 = require("schematics-utilities");
const angular_schematic_utils_1 = require("@objectivity/angular-schematic-utils");
function scssImport(options) {
    return (_tree, _context) => {
        return schematics_1.chain([
            updateAngularFile(options)
        ]);
    };
}
exports.scssImport = scssImport;
function updateAngularFile(options) {
    return (tree, _context) => {
        const angularConfig = getConfigPath(tree);
        const workspace = schematics_utilities_1.getWorkspace(tree);
        const projectName = options.project;
        const workspaceProject = schematics_utilities_1.getProjectFromWorkspace(workspace, projectName);
        let projectPath = angular_schematic_utils_1.trimSlashes(schematics_utilities_1.buildDefaultPath(workspaceProject));
        const pathToScssFolder = `${projectPath}/scss`;
        const stylePreprocessorOptions = {
            includePaths: [
                pathToScssFolder
            ]
        };
        addStylePreprocessorOptions(tree, angularConfig, pathToScssFolder, options.project, 'build', stylePreprocessorOptions);
        addStylePreprocessorOptions(tree, angularConfig, pathToScssFolder, options.project, 'test', stylePreprocessorOptions);
    };
}
function addStylePreprocessorOptions(tree, angularConfig, pathToScssFolder, project, architect, stylePreprocessorOptions) {
    const angularConfigContent = tree.read(angularConfig);
    if (!angularConfigContent) {
        throw new Error('Invalid path: ' + angularConfig);
    }
    const tsCfgAst = core_1.parseJsonAst(angularConfigContent.toString('utf-8'));
    if (tsCfgAst.kind !== 'object') {
        throw new Error('Invalid tsconfig content.');
    }
    const buildOptionsAstNode = findPropertyInDeepAstObject(tsCfgAst, ['projects', project, 'architect', architect, 'options']);
    const stylePreprocessorOptionsAstNode = schematics_utilities_1.findPropertyInAstObject(buildOptionsAstNode, 'stylePreprocessorOptions');
    if (!stylePreprocessorOptionsAstNode) {
        const recorder = tree.beginUpdate(angularConfig);
        schematics_utilities_1.appendPropertyInAstObject(recorder, buildOptionsAstNode, "stylePreprocessorOptions", stylePreprocessorOptions, 4);
        tree.commitUpdate(recorder);
    }
    else {
        if (stylePreprocessorOptionsAstNode.kind !== 'object') {
            throw new Error('Invalid stylePreprocessorOptionsAstNode content.');
        }
        const includePathsAstNode = schematics_utilities_1.findPropertyInAstObject(stylePreprocessorOptionsAstNode, 'includePaths');
        if (!includePathsAstNode) {
            const recorder = tree.beginUpdate(angularConfig);
            schematics_utilities_1.appendPropertyInAstObject(recorder, stylePreprocessorOptionsAstNode, 'includePaths', stylePreprocessorOptions.includePaths, 4);
            tree.commitUpdate(recorder);
        }
        else {
            if (includePathsAstNode && includePathsAstNode.kind != 'array') {
                throw new schematics_1.SchematicsException('Invalid includePaths property; expected an array.');
            }
            if (!includePathsAstNode.value.includes(pathToScssFolder)) {
                const recorder = tree.beginUpdate(angularConfig);
                schematics_utilities_1.appendValueInAstArray(recorder, includePathsAstNode, pathToScssFolder);
                tree.commitUpdate(recorder);
            }
        }
    }
}
function findPropertyInDeepAstObject(node, propertyNames) {
    propertyNames.forEach((property) => {
        const astNode = schematics_utilities_1.findPropertyInAstObject(node, property);
        if (!astNode || astNode.kind !== 'object') {
            throw new Error(`Invalid ${property} content.`);
        }
        node = astNode;
    });
    return node;
}
/*
function updateAngularFile(options: Schema): Rule {
    return (tree: Tree, _context: SchematicContext) => {
        const angularConfig = getConfigPath(tree);
        const rules: Rule[] = [];

        const angularConfigContent = tree.read(angularConfig);
        if (!angularConfigContent) {
            throw new Error('Invalid path: ' + angularConfig);
        }

        const angularJson = parseJson(angularConfigContent.toString(), JsonParseMode.Loose);

        if (!isJsonObject(angularJson) || !isJsonObject(angularJson.projects)) {
            // If that field isn't there, no use...
            return;
        }

        const project = angularJson.projects[options.project];

        if (!isJsonObject(project)) {
            return;
        }

        const targets = project.targets || project.architect;
        if (!isJsonObject(targets)) {
            return;
        }

        const target = targets['build'];
        if (isJsonObject(target)) {
          rules.push(_updateProjectTarget(target));
        }

        return chain(rules);
    };
}

function isJsonObject(value: JsonValue): value is JsonObject {
    return value != null && typeof value === 'object' && !Array.isArray(value);
  }
*/
function getConfigPath(tree) {
    const possiblePath = core_1.normalize('angular.json');
    if (tree.exists(possiblePath)) {
        return possiblePath;
    }
    throw new schematics_1.SchematicsException('Could not find configuration file');
}
/*
function _updateProjectTarget(targetObject: JsonObject): Rule {
    // Make sure we're using the correct builder.
    if (targetObject.builder !== '@angular-devkit/build-angular:browser'
        || !isJsonObject(targetObject.options)) {
      return noop();
    }
    const options = targetObject.options;
    if (!isJsonObject(targetObject.options.stylePreprocessorOptions)) {
        // Add to configuration
      return noop();
    }

    //const stylePreprocessorOptionsToUpdate = options.stylePreprocessorOptions;

    return noop();

    //const configurations = targetObject.configurations;
    // if (isJsonObject(configurations)) {
    //   for (const configName of Object.keys(configurations)) {
    //     const config = configurations[configName];

    //     // Just in case, only do non-AOT configurations.
    //     if (isJsonObject(config)
    //         && typeof config.polyfills == 'string'
    //         && config.aot !== true) {
    //       polyfillsToUpdate.push(config.polyfills);
    //     }
    //   }
    // }

    // return chain(
    //   polyfillsToUpdate.map(polyfillPath => {
    //     return (tree: Tree) => _removeReflectFromPolyfills(tree, polyfillPath);
    //   }),
    // );
  }
*/ 
//# sourceMappingURL=index.js.map